/*
	jwblackjack v1.0.1 by J. William Snyder, Jr.

	Originally written in OS9 Pascal during January and February 1987 
	as a high school programming class project.  It was inspired by a
	blackjack game that ran on a local BBS.

	Translated into C using p2c utility under Linux 2.0.2: September 1996.

	Copyright (C) 1999 J. William Snyder, Jr.
	Released under the terms of the GNU General Public License found in
        distribution archive.

	This software is released "AS IS" with NO WARRANTY WHATSOEVER.
	The implied warranties of merchantability and fitness for a
	particular purpose are expressly disclaimed.

	As of the date this archive was made, the distribution point for this
	program was http://metalab.unc.edu/jwsnyder/willhome.html.  If
	you have downloaded this game, I would appreciate an email: 
	jwsnyder@metalab.unc.edu

	p2c is required to compile this program, at least until I free it
	from its pascal ancestory.

	The following command should compile this program (worked for me under
        RedHat 6.0):

	gcc jwblackjack.c -ojwblackjack -lp2c -lm

	The rules are fairly simple:  Try to get a 21 without going over.
	The dealer hits on 16 and stands on 17.  Double-downs and
	five-and-unders are also supported.

	A file called CASHONHAND containing a real number representing the
	amount of your stake must be present in the directory in which
	jwblackjack resides in order for the game to run.  A sample CASHONHAND
	file is included with the distribution archive.
*/

#include <p2c/p2c.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

typedef Char STRING[2];
typedef STRING DECKARY[52];


Static DECKARY DECK, OLDECK;
Static long PCARDS, DCARDS;
Static FILE *fp;
Static float CASH, BET;
Static int NUMCARDS;
Static Char RESPOSE;
Static long CRDP, OLDP, DTOTAL, PTOTAL;
Static boolean FLAG, PUSH, DOUBLED, FIVEUD, FIVEUQ, BLCKJACK, PLAYERWIN, C876;


/*******************************/
Static Void SHUFFLE(DECK, NUMCARDS)
STRING *DECK;
int NUMCARDS;
{
  int Y, MO, D, H, M, S, COUNT, COUNT2;
  boolean FLAG;
  DECKARY CUTTER;
  STRING SUB1[26], SUB2[26];

  FLAG = false;
  printf("Shuffling...\n");
  S = abs(sin(time(NULL)) * 550);
  for (H = 1; H <= S; H++) {
    if (NUMCARDS & 1) {
      NUMCARDS--;
      FLAG = true;
    }
    for (COUNT = 0; COUNT < NUMCARDS / 2; COUNT++)
      memcpy(SUB1[COUNT], DECK[COUNT], sizeof(STRING));
    for (COUNT = NUMCARDS / 2; COUNT < NUMCARDS; COUNT++)
      memcpy(SUB2[COUNT - NUMCARDS / 2], DECK[COUNT], sizeof(STRING));
    COUNT = 1;
    COUNT2 = 1;
    while (COUNT2 < NUMCARDS) {
      memcpy(DECK[COUNT2 - 1], SUB2[COUNT - 1], sizeof(STRING));
      memcpy(DECK[COUNT2], SUB1[COUNT - 1], sizeof(STRING));
      COUNT++;
      COUNT2 += 2;
    }
    if (FLAG)
      NUMCARDS++;
    Y = abs(sin(time(NULL))) * 52;
    if (Y == 0)
      Y = 10;
    if (Y >= NUMCARDS)
      Y -= 10;
    for (MO = 1; MO <= Y; MO++)
      memcpy(CUTTER[NUMCARDS - labs(MO - Y) - 1], DECK[MO - 1],
	     sizeof(STRING));
    COUNT = 1;
    for (MO = Y; MO < NUMCARDS; MO++) {
      memcpy(CUTTER[COUNT - 1], DECK[MO], sizeof(STRING));
      COUNT++;
    }
    memcpy(DECK, CUTTER, sizeof(DECKARY));
  }
}


/*******************************/
Static Void MAKEVAR(DECK)
STRING *DECK;
{
  long COUNT, CT;
  Char LETTER;

  for (COUNT = 1; COUNT <= 13; COUNT++) {
    switch (COUNT) {

    case 1:
      LETTER = '2';
      break;

    case 2:
      LETTER = '3';
      break;

    case 3:
      LETTER = '4';
      break;

    case 4:
      LETTER = '5';
      break;

    case 5:
      LETTER = '6';
      break;

    case 6:
      LETTER = '7';
      break;

    case 7:
      LETTER = '8';
      break;

    case 8:
      LETTER = '9';
      break;

    case 9:
      LETTER = 'T';
      break;

    case 10:
      LETTER = 'J';
      break;

    case 11:
      LETTER = 'Q';
      break;

    case 12:
      LETTER = 'K';
      break;

    case 13:
      LETTER = 'A';
      break;
    }
    DECK[COUNT - 1][0] = LETTER;
    DECK[COUNT + 12][0] = LETTER;
    DECK[COUNT + 25][0] = LETTER;
    DECK[COUNT + 38][0] = LETTER;
  }
  for (CT = 0; CT <= 3; CT++) {
    switch (CT) {

    case 0:
      LETTER = 'S';
      break;

    case 1:
      LETTER = 'H';
      break;

    case 2:
      LETTER = 'C';
      break;

    case 3:   /*DUMMY*/
      LETTER = 'D';
      break;
    }
    for (COUNT = CT * 13; COUNT <= CT * 13 + 12; COUNT++)
      DECK[COUNT][1] = LETTER;

  }
}


typedef STRING ARY1[15];


Local Void TOT(TOTAL, CARD, CARDS, ACEUNDER)
long *TOTAL;
STRING *CARD;
long CARDS;
boolean *ACEUNDER;
{
  long COUNT, ACECOUNT;

  ACECOUNT = 0;
  *TOTAL = 0;
  *ACEUNDER = false;
  for (COUNT = 0; COUNT < CARDS; COUNT++) {
    switch (CARD[COUNT][0]) {

    case '2':
      *TOTAL += 2;
      break;

    case '3':
      *TOTAL += 3;
      break;

    case '4':
      *TOTAL += 4;
      break;

    case '5':
      *TOTAL += 5;
      break;

    case '6':
      *TOTAL += 6;
      break;

    case '7':
      *TOTAL += 7;
      break;

    case '8':
      *TOTAL += 8;
      break;

    case '9':
      *TOTAL += 9;
      break;

    case 'T':
    case 'J':
    case 'Q':
    case 'K':
      *TOTAL += 10;
      break;

    case 'A':
      ACECOUNT++;
      break;
    }
  }
  while (ACECOUNT != 0) {
    if (*TOTAL <= 10) {
      *TOTAL += 11;
      *ACEUNDER = true;
    } else
      (*TOTAL)++;
    ACECOUNT--;
  }
  if (*TOTAL > 21 && *ACEUNDER) {
    *ACEUNDER = false;
    *TOTAL -= 10;
  }
}

Local Void HIT(OLDECK, DECK, OLDP, CRDP, NUMCARDS, CARDS, MISTER)
STRING *OLDECK, *DECK;
long *OLDP, *CRDP, *NUMCARDS, *CARDS;
STRING *MISTER;
{
  if (*CRDP > *NUMCARDS) {
    memcpy(DECK, OLDECK, sizeof(DECKARY));
    *NUMCARDS = *OLDP - 1;
    SHUFFLE(DECK, *NUMCARDS);
    SHUFFLE(DECK, *NUMCARDS);
    *OLDP = 1;
    *CRDP = 1;
  }
  (*CARDS)++;
  memcpy(MISTER[*CARDS - 1], DECK[*CRDP - 1], sizeof(STRING));
  (*CRDP)++;
}


/*******************************/
Static Void PLAYGAME(DECK, OLDECK, PLAYERWIN, BLCKJACK, FIVEUD, FIVEUQ,
		     DOUBLED, PUSH, C876, CRDP, OLDP, NUMCARDS)
STRING *DECK, *OLDECK;
boolean *PLAYERWIN, *BLCKJACK, *FIVEUD, *FIVEUQ, *DOUBLED, *PUSH, *C876;
long *CRDP, *OLDP, *NUMCARDS;
{
  Char RESP;
  long PCARDS, DCARDS, TEMP, DTOTAL, PTOTAL;
  ARY1 DEALER, PLAYER;
  boolean PACEUNDER, DACEUNDER, PBUST, DBUST, TWOONE, DWINS;
  long COUNT, FORLIM;

  /*DEAL*/
  DWINS = false;
  *C876 = false;
  *DOUBLED = false;
  *BLCKJACK = false;
  *FIVEUD = false;
  *FIVEUQ = false;
  *PLAYERWIN = false;
  PBUST = false;
  DBUST = false;
  *PUSH = false;
  TWOONE = false;
  if (*CRDP > *NUMCARDS - 4) {
    FORLIM = *NUMCARDS;
    for (COUNT = *CRDP - 1; COUNT < FORLIM; COUNT++) {
      memcpy(OLDECK[*OLDP - 1], DECK[COUNT], sizeof(STRING));
      (*OLDP)++;
    }
    memcpy(DECK, OLDECK, sizeof(DECKARY));
    SHUFFLE(DECK, 52L);
    SHUFFLE(DECK, 52L);
    *CRDP = 1;
    *OLDP = 1;
    *NUMCARDS = 52;
  }
  memcpy(PLAYER[0], DECK[*CRDP - 1], sizeof(STRING));
  memcpy(PLAYER[1], DECK[*CRDP + 1], sizeof(STRING));
  memcpy(DEALER[0], DECK[*CRDP], sizeof(STRING));
  memcpy(DEALER[1], DECK[*CRDP + 2], sizeof(STRING));
  *CRDP += 4;
  PCARDS = 2;
  DCARDS = 2;
  TOT(&PTOTAL, PLAYER, PCARDS, &PACEUNDER);
  *BLCKJACK = (PTOTAL == 21);
  *PLAYERWIN = *BLCKJACK;
  do {
    TOT(&PTOTAL, PLAYER, PCARDS, &PACEUNDER);
    printf("--------------------------------\n");
    printf("Player:  ");
    for (TEMP = 0; TEMP < PCARDS; TEMP++)
      printf(" %.2s", PLAYER[TEMP]);
    if (PACEUNDER)
      printf(" (%2ld),(%2ld)\n", PTOTAL - 10, PTOTAL);
    else
      printf(" (%2ld)\n", PTOTAL);
    printf("Dealer:  ");
    printf(" %.2s ??\n", DEALER[0]);
    printf("--------------------------------\n");
    if (PTOTAL > 21) {
      PBUST = true;
      *PLAYERWIN = false;
      printf("You busted!!\n");
    }
    if (PTOTAL == 21 && !*BLCKJACK) {
      printf("You got a 21!\n");
      TWOONE = true;
      if (PCARDS == 3 && ((PLAYER[0][0] == '8' && PLAYER[1][0] == '7' &&
			   PLAYER[2][0] == '6') ||
			  (PLAYER[0][0] == '8' && PLAYER[1][0] == '6' &&
			   PLAYER[2][0] == '7') ||
			  (PLAYER[0][0] == '7' && PLAYER[1][0] == '8' &&
			   PLAYER[2][0] == '6') ||
			  (PLAYER[0][0] == '7' && PLAYER[1][0] == '6' &&
			   PLAYER[2][0] == '8') ||
			  (PLAYER[0][0] == '6' && PLAYER[1][0] == '8' &&
			   PLAYER[2][0] == '7') ||
			  (PLAYER[0][0] == '6' && PLAYER[1][0] == '8' &&
			   PLAYER[2][0] == '7'))) {
/* p2c: blakjack.pas, line 377: 
 * Note: Line breaker spent 0.0 seconds, 5000 tries on line 346 [251] */
	printf("with an 8-7-6 combination!\n");
	*C876 = true;
	*PLAYERWIN = true;
      }
    }
    if (PCARDS == 6 && !PBUST) {
      printf("You got a 5 and under quadruple bet!!\n");
      *PLAYERWIN = true;
    }
    if (PCARDS == 5 && !PBUST) {
      *FIVEUD = true;
      *PLAYERWIN = true;
      printf("You got a 5 and under!!\n");
      printf("You may:\n");
      printf("<S>tand and win a double bet.\n");
      printf("<H>it and try for a quadruple bet.\n");
      do {
	printf("What will it be?:");
	fflush(stdout);
	scanf("%c%*[^\n]", &RESP);
	getchar();
	if (RESP == '\n')
	  RESP = ' ';
	putchar('\n');
      } while (RESP != 'H' && RESP != 'S' && RESP != 'h' && RESP != 's');
      if (RESP == 'H' || RESP == 'h') {
	HIT(OLDECK, DECK, OLDP, CRDP, NUMCARDS, &PCARDS, PLAYER);
	*FIVEUQ = true;
	*FIVEUD = false;
	*PLAYERWIN = false;
      }
    }
    if (*BLCKJACK)
      printf("You got a blackjack!!\n");
    if (*PLAYERWIN == false && PBUST == false && *FIVEUQ == false &&
	*FIVEUD == false && TWOONE == false && *DOUBLED == false &&
	*C876 == false) {
      if (PTOTAL == 11 && PCARDS == 2) {
	do {
	  printf("<H>it <S>tand <D>ouble?:");
	  fflush(stdout);
	  scanf("%c%*[^\n]", &RESP);
	  getchar();
	  if (RESP == '\n')
	    RESP = ' ';
	  putchar('\n');
	} while (RESP != 'H' && RESP != 'S' && RESP != 'D' && RESP != 'h' && 
RESP != 's' && RESP != 'd');
	*DOUBLED = (RESP == 'D' || RESP == 'd');
      } else {
	do {
	  printf("<H>it <S>tand?:");
	  fflush(stdout);
	  scanf("%c%*[^\n]", &RESP);
	  getchar();
	  if (RESP == '\n')
	    RESP = ' ';
	  putchar('\n');
	} while (RESP != 'H' && RESP != 'S' && RESP != 'h' && RESP != 's');
      }
      if (RESP == 'D' || RESP == 'H' || RESP == 'd' || RESP == 'h')
	HIT(OLDECK, DECK, OLDP, CRDP, NUMCARDS, &PCARDS, PLAYER);
    }
    TOT(&PTOTAL, PLAYER, PCARDS, &PACEUNDER);
  } while (!(*BLCKJACK || *FIVEUD || *DOUBLED || TWOONE || PBUST ||
	     *PLAYERWIN || *C876 || RESP == 'S' || RESP == 's'));
  if (!(*PLAYERWIN || PBUST)) {
    TOT(&DTOTAL, DEALER, DCARDS, &DACEUNDER);
    do {
      TOT(&DTOTAL, DEALER, DCARDS, &DACEUNDER);
      printf("--------------------------------\n");
      printf("Player:  ");
      for (TEMP = 0; TEMP < PCARDS; TEMP++)
	printf(" %.2s", PLAYER[TEMP]);
      if (PACEUNDER)
	printf(" (%2ld),(%2ld)\n", PTOTAL - 10, PTOTAL);
      else
	printf(" (%2ld)\n", PTOTAL);
      printf("Dealer:  ");
      for (TEMP = 0; TEMP < DCARDS; TEMP++)
	printf(" %.2s", DEALER[TEMP]);
      if (DACEUNDER)
	printf(" (%2ld),(%2ld)\n", DTOTAL - 10, DTOTAL);
      else
	printf(" (%2ld)\n", DTOTAL);
      printf("--------------------------------\n");
      if (DTOTAL > 21) {
	printf("The dealer busted!!\n");
	DBUST = true;
	*PLAYERWIN = true;
      }
      if (DTOTAL == 21 && DCARDS == 2) {
	printf("The dealer gets a blackjack!!\n");
	DWINS = true;
      }
      if (DTOTAL < 17)
	HIT(OLDECK, DECK, OLDP, CRDP, NUMCARDS, &DCARDS, DEALER);
      else {
	if (DTOTAL < PTOTAL) {
	  printf("You win!!\n");
	  *PLAYERWIN = true;
	}
	if (DTOTAL == PTOTAL && !DWINS) {
	  printf("Push.\n");
	  *PUSH = true;
	}
	if (DTOTAL > PTOTAL && !DBUST) {
	  printf("The dealer wins!!\n");
	  DWINS = true;
	}
      }
      for (COUNT = 1; COUNT <= 5000; COUNT++) ;
    } while (!(*PLAYERWIN || DWINS || *PUSH));
  }
  /*CLEAN UP*/
  for (COUNT = 0; COUNT < PCARDS; COUNT++) {
    memcpy(OLDECK[*OLDP - 1], PLAYER[COUNT], sizeof(STRING));
    (*OLDP)++;
  }
  for (COUNT = 0; COUNT < DCARDS; COUNT++) {
    memcpy(OLDECK[*OLDP - 1], DEALER[COUNT], sizeof(STRING));
    (*OLDP)++;
  }
}


/*******************************/
/*MAIN PROGRAM MODULE*/
main(argc, argv)
int argc;
Char *argv[];
{
  PASCAL_MAIN(argc, argv);
  if ((fp  = fopen ("CASHONHAND", "r+")) == NULL )
	{
		fprintf(stderr, "Error opening CASHONHAND");
		exit(1);
	}
/*  CASHONHAND = NULL;*/
  printf("\f\n");
  printf("jwblackjack 1.0.1 by J. William Snyder, Jr.  September 1996\n");
  fscanf(fp, "%f", &CASH);
  NUMCARDS = 52;
  MAKEVAR(DECK);
  SHUFFLE(DECK, 52L);
  SHUFFLE(DECK, 52L);
  CRDP = 1;
  OLDP = 1;
  BET = 0.0;
  do {
    do {
      printf("Cash on hand: $%10.2f\n", CASH);
      printf("Your bet (0=quit):");
      fflush(stdout);
      scanf("%f", &BET);
      getchar();
      putchar('\n');
    } while (BET > CASH);
    if (BET > 0) {
      PLAYGAME(DECK, OLDECK, &PLAYERWIN, &BLCKJACK, &FIVEUD, &FIVEUQ, &DOUBLED,
	       &PUSH, &C876, &CRDP, &OLDP, &NUMCARDS);
      if (BLCKJACK)
	BET *= 1.5;
      if (FIVEUD || DOUBLED || C876)
	BET *= 2;
      if (FIVEUQ)
	BET *= 4;
      if (PUSH)
	BET = 0.0;
      if (PLAYERWIN)
	CASH += BET;
      else
	CASH -= BET;
    }
    if (PUSH)
      BET = 1.0;
  } while (BET > 0 && CASH > 0);
  if (fp != NULL)
    rewind(fp);
  else
    fp = tmpfile();
  if (fp == NULL)
    _EscIO(FileNotFound);
  fprintf(fp, "% .5f\n", CASH);
  if (CASH == 0)
    printf("Sorry, but you're broke!\n");
  printf("Thank you for playing jwblackjack!!\n");
  if (fp != NULL)
    fclose(fp);
  exit(EXIT_SUCCESS);
}



/* End. */

